{**********************************************************************
          Copyright (c) PilotLogic Software House
                   All rights reserved

 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)
 ct9999

 Contributors:
              Forum User @saulo

***********************************************************************}

// for Embedded Form Designer

unit ct_emb_formeditor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, ExtCtrls, Buttons,
  Math,
  IDEWindowIntf, Toolwin,
  Graphics, Dialogs,
  ComCtrls, LMessages,
  LCLIntf,
  LCLType,
  LCLClasses, dateutils, types, typinfo,
  ctvarside,
  LazLoggerBase, ct_idestrconsts,
  intfgraphics,LazIDEIntf, IDEImagesIntf, StdCtrls,  EditBtn, Menus, ComponentEditors,
  SourceSynEditor,
  ct_emb_basics, ct_emb_formlist, ct_embedded_designer_settings;

type

TEmbedHandlerOption=(ehoSendBefore,ehoSendAfter,ehoSendToForm);
TEmbedHandlerOptions=Set of TEmbedHandlerOption;

TFakeFormRec=Record
    aBorderStyle : TFormBorderStyle;
    aBIMaximize  : Boolean;
    aBIMinimize  : Boolean;
    aBISysMenu   : Boolean;
end;

TEmbedTabSheet = class;
TEmbedFormEditor = class;

TEmbedMessageHandler = class(TPersistent)
  protected
    FHookControl: TControl;
    InheritedWndMethod: TWndMethod;
  public
    FOptions:TEmbedHandlerOptions;
    OnBeforeMessage,
    OnAfterMessage  : TEmbedMessageHandlerEvent;
    constructor Create;
    destructor Destroy; override;
    procedure HookControl(AControl: TControl);
    procedure UnHookControl;
    function  HasHookControl: Boolean;
    procedure SelfWndMethod(var TheMessage: TLMessage);
    Procedure SetDefaulOptions;
    Property  TheHookControl: TControl read FHookControl;
  end;

TTEmbedFakeMainMenu = class(TToolBar)
  protected
    FMainMenu:TMainMenu;
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    Procedure MainMenuAssign(aMainMenu:TMainMenu);
    Procedure MainMenuClear;
    Procedure MainMenuUpdate;
end;

{ TEmbedDesignPanel }

TEmbedDesignPanel = class(TPanel)
  protected
    FHasLoadedForm: Boolean;
    FHasStoredForm: Boolean;

    FLoadedForm: TCustomForm;
    FLoadedFormType:TEmbedFormType;
    FFakeWindow: TPanel;
    FFakeWindow_MouseDown: Boolean;
    FFakeWindow_MousePos: TPoint;
    FFakeWindowTitle: TPanel;
    FFormMainMenuHeight:Integer;  //789: Fix for TMainMenu + TStatusBar Problem
    FMessagesHandler:TEmbedMessageHandler;
    FFakeMainMenu:TTEmbedFakeMainMenu;
    FMouseArea:integer;
    FExecuteBeforeMessage: Boolean;
    FIniOK:boolean;
    OnFormLoad: TNotifyEvent;
    OnLoadedFormChangeBounds: TNotifyEvent;

    FZeroOriginPoint:Tpoint;
    FDeltaPointMousePos: TPoint;

    dsParentPage:TEmbedTabSheet;
    dsParentFormEditor: TEmbedFormEditor;

    //FakeButtons
    FFakeIcon       : TImage;
    FFakeCaption    : TLabel;
    FFakeBtnSystem  : TSpeedButton;
    FFakeBtnMinimize: TSpeedButton;
    FFakeBtnMaximize: TSpeedButton;
    FLastLFormRec   : TFakeFormRec;

    //Resizers
    FResizerRight        : TImage;
    FResizerBottom       : TImage;
    FResizerCornerRight  : TImage;
    FResizerCornerBottom : TImage;

    FResizerHighlight    : Boolean;
    FResizerHighlightTag : Integer;

    procedure InitializeWnd; override;
    function  FormIsValid(aForm: TCustomForm): Boolean;
    function  LoadedFormIsValid: Boolean;
    function  FormZeroOrigin(aForm: TCustomForm): TPoint;
    Function  CheckForFakeMainMenu(aForm: TCustomForm):Boolean;
    Procedure UpdateFakeMainMenu;
    procedure AddForm(aForm: TCustomForm);
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure EmbeddedForm(Sender: TObject; aForm: TCustomForm);
    procedure UnLoadLoadedForm;

    procedure onForm_BeforeMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
    procedure onForm_AfterMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
    // FakeWindow
    procedure onFakeWindowMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure onFakeWindowResize(Sender: TObject);
    procedure onFakeCaptionPaint(Sender: TObject);
    // FakeButtons
    procedure FakeButtonsDraw(Const ForceDraw:Boolean);
    Procedure FakeIconDrawIcon;
    // Resizers
    procedure onResizerMouseEnter(Sender: TObject);
    procedure onResizerMouseLeave(Sender: TObject);
    procedure onResizerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure onResizerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure onResizerPaint(sender: Tobject);
    procedure ResizersDraw(Sender: TObject);
    procedure ResizersUpdate;

   // Execute Commands to Loaded Form like:
   // LM_ACTIVATE, LM_SETFOCUS, LM_MOVE, LM_SIZE, CM_SHOWINGCHANGED, etc
    procedure ExecFormCommand(Const aCommand: UINT);

    property IniOK:boolean read FIniOK write FIniOK;
    property LoadedForm: TCustomForm read FLoadedForm;
    property LoadedFormType:TEmbedFormType read FLoadedFormType;
  end;

{ TEmbedFormEditor }

TEmbedFormEditor = class(TPanel)
  private
    imgWindowScreenPreview: TImage;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelW: TLabel;
    LabelH: TLabel;
    pnShowHide: TPanel;
    RootPanel: TPanel;
    pnInfoPanel: TPanel;
    pnScreenPreviewBtns: TPanel;
    pnCaptionWindowScreenPreview: TPanel;
    pnScreenPreview: TPanel;
    pnWindowScreenPreview: TPanel;
    pnDesignBox: TScrollBox;
    sb_hide: TSpeedButton;
    sb_SetUp: TSpeedButton;
    sb_zero: TSpeedButton;
    sb_up: TSpeedButton;
    sb_bottom: TSpeedButton;
    sb_left: TSpeedButton;
    sb_right: TSpeedButton;
    sb_update: TSpeedButton;
    procedure imgWindowScreenPreviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure imgWindowScreenPreviewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure imgWindowScreenPreviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure sb_upClick(Sender: TObject);
    procedure sb_updateClick(Sender: TObject);
    procedure sb_hideClick(Sender: TObject);
    procedure sb_SetUpClick(Sender: TObject);
  private
    FPreviewMoving: boolean;
    FPreviewMovingPoint: TPoint;
    FDesignAreaViewMoving: boolean;
    FDesignAreaViewMovingPoint: TPoint;
  public
    FDesignPanel: TEmbedDesignPanel;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure InfoPanelDraw(Const aRect:TRect);
    function  PixelPerc(vl, perc: single): integer;
    procedure DesignPanel_OnLoadForm(Sender: TObject);
    procedure DesignPanel_OnLoadedFormChangeBounds(Sender: TObject);
    procedure SetupPreviewFormDesign(_Form: TCustomForm);
    Procedure ActivatedDesigner;
    Procedure UpdateALLVisibleStaff;
  end;

{ TEmbedTabSheet }

TEmbedTabSheet = class(TTabSheet)
  private
    FLoadedFormSourceWinNum : Integer;     // for Info NOT use this
    FLoadedFormEmbedTabSheetNum : Integer; // for Info NOT use this
    FLoadedFormFileName:string;            // for Info NOT use this
    FLoadedFormClassName:string;           // for Info NOT use this

    FOnToggleFormOrSource:TNotifyEvent;
    Function GetLoadedForm:TCustomForm;
  public
    FEditPages : TPageControl;
    FEditPageForSource : TTabSheet;
    FEditPageForDesigner : TTabSheet;

    FFormEditor: TEmbedFormEditor;
    FSynEditor : TIDESynEditor;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure DoOnTabControlChange(Sender: TObject);

    procedure ShowSourceTab;
    procedure ShowFormEditorTab;

    procedure EditPages_Create;
    procedure EditPages_Free;

    procedure FormEditorsINIT;
    Procedure FormEditorDELETE;
    Procedure FormEditorStore;
    Procedure FormEditorUnStore;

    Procedure FormEditorAdd;
    Function  EmbeddedForm(Sender: TObject; aForm: TCustomForm; Const aWin,aETS:Integer; Const aFCN,aFile:String):Boolean;

    Procedure UpdateEmbedFormDsgSettings;

    property LoadedForm: TCustomForm read GetLoadedForm;
    property LoadedFormSourceWinNum: Integer read FLoadedFormSourceWinNum write FLoadedFormSourceWinNum;
    property LoadedFormEmbedTabSheetNum: Integer read FLoadedFormEmbedTabSheetNum write FLoadedFormEmbedTabSheetNum;
    property LoadedFormClassName:  string read FLoadedFormClassName write FLoadedFormClassName;
    property LoadedFormFileName: string read FLoadedFormFileName write FLoadedFormFileName;

    property OnToggleFormOrSource:TNotifyEvent read FOnToggleFormOrSource write FOnToggleFormOrSource;
  end;

  procedure ProcEmb_FormShow(aForm: TCustomForm);
  procedure ProcEmb_FormHide(aForm: TCustomForm);

var
  EMBEDDED_FORM_DESIGNER: PLazLoggerLogGroup;

implementation


//==============================================================================

procedure ProcEmb_FormShow(aForm: TCustomForm);
begin
  if (aForm = Nil) or (not Assigned(aForm)) then Exit;

  //... ct5555 .................................
  // Solve mouse events Problem on Cocoa
  {$IF DEFINED(LCLCocoa) or DEFINED(LCLGtk3) or DEFINED(LCLGtk4)}
    LCLIntf.ShowWindow(aForm.Handle, SW_SHOWNORMAL);
    aForm.Visible:=True;
  {$ELSE}
    LCLIntf.ShowWindow(aForm.Handle, SW_SHOWNORMAL);
  {$ENDIF}
  //...........................................
end;

procedure ProcEmb_FormHide(aForm: TCustomForm);
begin
  if (aForm = Nil) or (not Assigned(aForm)) then Exit;

//... ct5555 .................................
// Solve Form.Caption Problem on Cocoa
{$IF DEFINED(LCLCocoa) or DEFINED(LCLGtk3) or DEFINED(LCLGtk4)}
  aform.Visible:=false;
{$ELSE}
  LCLIntf.ShowWindow(aForm.Handle, SW_HIDE);
{$ENDIF}
//............................................
end;

//==============================================================================
//=========================== TEmbedFormEditor =================================
//==============================================================================
constructor TEmbedFormEditor.Create(TheOwner: TComponent);
begin
 Inherited;

  self.Left   := 0;
  self.Height := 434;
  self.Top    := 0;
  self.Width  := 545;
  self.Align  := alClient;
  self.BevelOuter := bvNone;
  self.ClientHeight := 434;
  self.ClientWidth  := 545;
  self.Color := clDefault;
  self.ParentColor := False;
  self.TabOrder := 0;

  //...............................
    RootPanel:= TPanel.Create(Self);
    RootPanel.Parent:=self;
    RootPanel.Left := 0;
    RootPanel.Height := 434;
    RootPanel.Top := 0;
    RootPanel.Width := 545;
    RootPanel.Align := alClient;
    RootPanel.BevelOuter := bvNone;
    RootPanel.ClientHeight := 434;
    RootPanel.ClientWidth := 545;
    RootPanel.DoubleBuffered := False;
    RootPanel.ParentColor := False;
    RootPanel.ParentDoubleBuffered := False;
    RootPanel.TabOrder := 0;

      pnDesignBox:= TScrollBox.Create(RootPanel);
      pnDesignBox.Parent:=RootPanel;
      pnDesignBox.Align := alClient;
      pnDesignBox.Left := 0;
      pnDesignBox.Height := 434;
      pnDesignBox.Top := 0;
      pnDesignBox.Width := 545;
      pnDesignBox.HorzScrollBar.Increment := 1;
      pnDesignBox.HorzScrollBar.Page := 1;
      pnDesignBox.HorzScrollBar.Smooth := True;
      pnDesignBox.HorzScrollBar.Tracking := True;
      pnDesignBox.VertScrollBar.Increment := 1;
      pnDesignBox.VertScrollBar.Page := 1;
      pnDesignBox.VertScrollBar.Smooth := True;
      pnDesignBox.VertScrollBar.Tracking := True;
      pnDesignBox.BorderStyle := bsNone;
      pnDesignBox.Color := clWindow;
      pnDesignBox.ParentColor := False;
      pnDesignBox.TabOrder := 0;

      //..............................................
      pnScreenPreview:= TPanel.Create(RootPanel);
      pnScreenPreview.Parent:=RootPanel;
      pnScreenPreview.AnchorSideRight.Side := asrBottom;
      pnScreenPreview.AnchorSideBottom.Side := asrBottom;
      pnScreenPreview.Left := 384;
      pnScreenPreview.Top := 352;
      pnScreenPreview.Width  := PixelPerc(Screen.Width,  CN_PREVIEW_FACTOR);
      pnScreenPreview.Height := PixelPerc(Screen.Height, CN_PREVIEW_FACTOR);
      pnScreenPreview.BorderSpacing.Right := 3;
      pnScreenPreview.BorderSpacing.Bottom := 3;
      pnScreenPreview.BevelInner := bvLowered ;
      pnScreenPreview.ClientWidth  := pnScreenPreview.Width;
      pnScreenPreview.ClientHeight := pnScreenPreview.Height;
      pnScreenPreview.Color := 10841658;
      pnScreenPreview.DoubleBuffered := False;
      pnScreenPreview.ParentColor := False;
      pnScreenPreview.ParentDoubleBuffered := False;
      pnScreenPreview.ParentShowHint := False;
      pnScreenPreview.ShowHint := True;
      pnScreenPreview.TabOrder := 1;
      pnScreenPreview.Anchors := [akRight, akBottom];
      pnScreenPreview.AnchorSideRight.Control := RootPanel;
      pnScreenPreview.AnchorSideRight.Side := asrBottom;
      pnScreenPreview.BorderSpacing.Right  := 4+LCLIntf.GetSystemMetrics(SM_CYVSCROLL);
      pnScreenPreview.AnchorSideBottom.Control := RootPanel;
      pnScreenPreview.AnchorSideBottom.Side := asrBottom;
      pnScreenPreview.BorderSpacing.Bottom := 4+LCLIntf.GetSystemMetrics(SM_CXHSCROLL);

         pnWindowScreenPreview:= TPanel.Create(pnScreenPreview);
         pnWindowScreenPreview.Parent:=pnScreenPreview;
         pnWindowScreenPreview.Left := CN_BORDER_PX;
         pnWindowScreenPreview.Top := CN_BORDER_PX;
         pnWindowScreenPreview.Height := 35;
         pnWindowScreenPreview.Width := 50;
         pnWindowScreenPreview.ClientHeight := 35;
         pnWindowScreenPreview.ClientWidth := 50;
         pnWindowScreenPreview.Color := clForm;
         pnWindowScreenPreview.ParentColor := False ;
         pnWindowScreenPreview.TabOrder := 0 ;

            pnCaptionWindowScreenPreview:= TPanel.Create(pnWindowScreenPreview);
            pnCaptionWindowScreenPreview.Parent:=pnWindowScreenPreview;
            pnCaptionWindowScreenPreview.Left := 1;
            pnCaptionWindowScreenPreview.Height := 3 ;
            pnCaptionWindowScreenPreview.Top := 1 ;
            pnCaptionWindowScreenPreview.Width := 48  ;
            pnCaptionWindowScreenPreview.Align := alTop;
            pnCaptionWindowScreenPreview.BevelOuter := bvNone ;
            pnCaptionWindowScreenPreview.Color := clActiveCaption;
            pnCaptionWindowScreenPreview.ParentColor := False ;
            pnCaptionWindowScreenPreview.TabOrder := 0;
            pnCaptionWindowScreenPreview.OnMouseDown := @imgWindowScreenPreviewMouseDown ;
            pnCaptionWindowScreenPreview.OnMouseMove := @imgWindowScreenPreviewMouseMove ;
            pnCaptionWindowScreenPreview.OnMouseUp   := @imgWindowScreenPreviewMouseUp ;

            imgWindowScreenPreview:= TImage.Create(pnWindowScreenPreview);
            imgWindowScreenPreview.Parent:=pnWindowScreenPreview;
            imgWindowScreenPreview.Cursor := crCross ;
            imgWindowScreenPreview.Left := 1 ;
            imgWindowScreenPreview.Height := 30 ;
            imgWindowScreenPreview.Top := 4 ;
            imgWindowScreenPreview.Width := 48 ;
            imgWindowScreenPreview.Align := alClient ;
            imgWindowScreenPreview.Stretch := True;
            imgWindowScreenPreview.OnMouseDown := @imgWindowScreenPreviewMouseDown;
            imgWindowScreenPreview.OnMouseMove := @imgWindowScreenPreviewMouseMove;
            imgWindowScreenPreview.OnMouseUp   := @imgWindowScreenPreviewMouseUp;

      //...........................................
      pnScreenPreviewBtns:= TPanel.Create(RootPanel);
      pnScreenPreviewBtns.Parent:=RootPanel;
      pnScreenPreviewBtns.Left   := 384;
      pnScreenPreviewBtns.Top    := 256;
      pnScreenPreviewBtns.Width  := 80;
      pnScreenPreviewBtns.Height := 72;
      pnScreenPreviewBtns.ClientWidth  := 80;
      pnScreenPreviewBtns.ClientHeight := 72;
      pnScreenPreviewBtns.Anchors := [akRight, akBottom];
      pnScreenPreviewBtns.BevelOuter := bvNone;
      pnScreenPreviewBtns.DoubleBuffered := False;
      pnScreenPreviewBtns.ParentDoubleBuffered := False;
      pnScreenPreviewBtns.TabOrder := 2;
      pnScreenPreviewBtns.Anchors := [akRight, akBottom];
      pnScreenPreviewBtns.AnchorSideRight.Control := RootPanel;
      pnScreenPreviewBtns.AnchorSideRight.Side := asrBottom;
      pnScreenPreviewBtns.BorderSpacing.Right  := 30 +LCLIntf.GetSystemMetrics(SM_CYVSCROLL);
      pnScreenPreviewBtns.AnchorSideBottom.Control := pnScreenPreview;
      pnScreenPreviewBtns.AnchorSideBottom.Side := asrTop;
      pnScreenPreviewBtns.BorderSpacing.Bottom := 4;

        sb_up:= TSpeedButton.Create(pnScreenPreviewBtns);
        sb_up.Parent:=pnScreenPreviewBtns;
        sb_up.Left  := 30;
        sb_up.Top   := 4;
        sb_up.Width := 23;
        sb_up.Height:= 22;
        sb_up.Anchors := [akTop, akRight];
        sb_up.ShowHint := True;
        sb_up.ParentShowHint := False;
        sb_up.Transparent:=True;
        sb_up.Flat:=True;
        sb_up.Hint:=lisEFDup;
        IDEImages.AssignImage(sb_up,'ct_moveu');
        sb_up.OnClick := @sb_upClick;

        sb_bottom:= TSpeedButton.Create(pnScreenPreviewBtns);
        sb_bottom.Parent:=pnScreenPreviewBtns;
        sb_bottom.Left  := 30;
        sb_bottom.Top   := 48;
        sb_bottom.Width := 23;
        sb_bottom.Height:= 22;
        sb_bottom.Anchors := [akTop, akRight];
        sb_bottom.ShowHint := True ;
        sb_bottom.ParentShowHint := False ;
        sb_bottom.Transparent:=True;
        sb_bottom.Flat:=True;
        sb_bottom.Hint:=lisEFDbottom;
        IDEImages.AssignImage(sb_bottom,'ct_moved');
        sb_bottom.OnClick := @sb_upClick ;

        sb_left:= TSpeedButton.Create(pnScreenPreviewBtns);
        sb_left.Parent:=pnScreenPreviewBtns;
        sb_left.Left  := 4;
        sb_left.Top   := 26;
        sb_left.Width := 23;
        sb_left.Height:= 22;
        sb_left.Anchors := [akTop, akRight];
        sb_left.ShowHint := True ;
        sb_left.ParentShowHint := False;
        sb_left.Transparent:=True;
        sb_left.Flat:=True;
        sb_left.Hint:=lisEFDleft;
        IDEImages.AssignImage(sb_left,'ct_movel');
        sb_left.OnClick := @sb_upClick;

        sb_right:= TSpeedButton.Create(pnScreenPreviewBtns);
        sb_right.Parent:=pnScreenPreviewBtns;
        sb_right.Left  := 54;
        sb_right.Top   := 26;
        sb_right.Width := 23;
        sb_right.Height:= 22;
        sb_right.Anchors := [akTop, akRight];
        sb_right.ShowHint := True;
        sb_right.ParentShowHint := False ;
        sb_right.Transparent:=True;
        sb_right.Flat:=True;
        sb_right.Hint :=lisEFDright;
        IDEImages.AssignImage(sb_right,'ct_mover');
        sb_right.OnClick := @sb_upClick ;

        sb_zero:= TSpeedButton.Create(pnScreenPreviewBtns);
        sb_zero.Parent:=pnScreenPreviewBtns;
        sb_zero.Left  := 30;
        sb_zero.Top   := 26;
        sb_zero.Width := 23;
        sb_zero.Height:= 22;
        sb_zero.Anchors := [akTop, akRight] ;
        sb_zero.ShowHint := True;
        sb_zero.ParentShowHint := False;
        sb_zero.Transparent:=True;
        sb_zero.Flat:=True;
        sb_zero.Hint:=lisEFDzero;
        IDEImages.AssignImage(sb_zero,'ct_reset');
        sb_zero.OnClick := @sb_upClick ;

     //...........................................
      pnInfoPanel:= TPanel.Create(RootPanel);
      pnInfoPanel.Parent:=RootPanel;
      pnInfoPanel.Left := 432;
      pnInfoPanel.Top  := 16;
      pnInfoPanel.Width  := 84;
      pnInfoPanel.Height := 84;
      pnInfoPanel.ClientHeight := 84;
      pnInfoPanel.ClientWidth  := 84;
      pnInfoPanel.BevelInner := bvLowered;
      pnInfoPanel.Color := clWindow;
      pnInfoPanel.Font.Color := clWindowText;
      pnInfoPanel.ParentColor := False;
      pnInfoPanel.ParentFont := False;
      pnInfoPanel.TabOrder := 3 ;
      pnInfoPanel.Visible := False;
      pnInfoPanel.Anchors := [akTop, akRight] ;
      pnInfoPanel.AnchorSideRight.Control := RootPanel;
      pnInfoPanel.AnchorSideRight.Side := asrBottom;
      pnInfoPanel.BorderSpacing.Right  := 4 +LCLIntf.GetSystemMetrics(SM_CYVSCROLL);
      pnInfoPanel.AnchorSideTop.Control := RootPanel;
      pnInfoPanel.BorderSpacing.Top := 4;
      pnInfoPanel.AnchorSideBottom.Control := RootPanel;
      pnInfoPanel.AnchorSideBottom.Side := asrCenter;

        LabelX:= TLabel.Create(pnInfoPanel);
        LabelX.Parent:=pnInfoPanel;
        LabelX.Left  := 8;
        LabelX.Top   := 8;
        LabelX.Width := 10;
        LabelX.Height:= 15;
        LabelX.Caption := 'X:';
        LabelX.ParentColor := False;

        LabelY:= TLabel.Create(pnInfoPanel);
        LabelY.Parent:=pnInfoPanel;
        LabelY.Left  := 8;
        LabelY.Top   := 24;
        LabelY.Width := 10 ;
        LabelY.Height:= 15;
        LabelY.Caption := 'Y:';
        LabelY.ParentColor := False;

        LabelW:= TLabel.Create(pnInfoPanel);
        LabelW.Parent:=pnInfoPanel;
        LabelW.Left  := 8;
        LabelW.Top   := 40;
        LabelW.Width := 14;
        LabelW.Height:= 15;
        LabelW.Caption := 'W:';
        LabelW.ParentColor := False;

        LabelH:= TLabel.Create(pnInfoPanel);
        LabelH.Parent:=pnInfoPanel;
        LabelH.Left  := 8;
        LabelH.Top   := 56;
        LabelH.Width := 12;
        LabelH.Height:= 15;
        LabelH.Caption := 'H:';
        LabelH.ParentColor := False ;

   //..........................................
    pnShowHide:= TPanel.Create(RootPanel);
    pnShowHide.Parent:=RootPanel;
    pnShowHide.Left   := 171;
    pnShowHide.Top    := 256;
    pnShowHide.Width  := 24;
    pnShowHide.Height := 76;
    pnShowHide.ClientWidth  := 24;
    pnShowHide.ClientHeight := 76;
    pnShowHide.BevelOuter := bvNone;
    pnShowHide.DoubleBuffered := False;
    pnShowHide.ParentDoubleBuffered := False;
    pnShowHide.TabOrder := 1;
    pnShowHide.Anchors := [akRight, akBottom];
    pnShowHide.AnchorSideRight.Control := RootPanel;
    pnShowHide.AnchorSideRight.Side := asrBottom;
    pnShowHide.BorderSpacing.Right  := 4+LCLIntf.GetSystemMetrics(SM_CYVSCROLL);
    pnShowHide.AnchorSideBottom.Control := RootPanel;
    pnShowHide.AnchorSideBottom.Side := asrBottom;
    pnShowHide.BorderSpacing.Bottom := 6+LCLIntf.GetSystemMetrics(SM_CYHSCROLL)+PixelPerc(Screen.Height, CN_PREVIEW_FACTOR);

      sb_hide:= TSpeedButton.Create(pnShowHide);
      sb_hide.Parent:=pnShowHide;
      sb_hide.Left  := 1;
      sb_hide.Top   := 2;
      sb_hide.Width := 23;
      sb_hide.Height:= 22;
      sb_hide.Anchors := [akRight, akBottom];
      sb_hide.Flat := True;
      sb_hide.ShowHint := True;
      sb_hide.ParentShowHint := False;
      sb_hide.Transparent:=True;
      sb_hide.Hint:=lisEFDhide;
      IDEImages.AssignImage(sb_hide,'ct_show');
      sb_hide.OnClick := @sb_hideClick;

      sb_update:= TSpeedButton.Create(pnShowHide);
      sb_update.Parent:=pnShowHide;
      sb_update.Left  := 1;
      sb_update.Top   := 28;
      sb_update.Width := 23;
      sb_update.Height:= 22;
      sb_update.Anchors := [akRight, akBottom];
      sb_update.Flat:=True;
      sb_update.ShowHint := True;
      sb_update.ParentShowHint := False;
      sb_update.Transparent:=True;
      sb_update.Hint:=lisEFDupdate;
      IDEImages.AssignImage(sb_update,'ct_update');
      sb_update.OnClick := @sb_updateClick;

      sb_SetUp:= TSpeedButton.Create(pnShowHide);
      sb_SetUp.Parent:=pnShowHide;
      sb_SetUp.Left  := 1;
      sb_SetUp.Top   := 54;
      sb_SetUp.Width := 23;
      sb_SetUp.Height:= 22;
      sb_SetUp.Anchors := [akRight, akBottom];
      sb_SetUp.Flat := True;
      sb_SetUp.ShowHint := True;
      sb_SetUp.ParentShowHint := False;
      sb_SetUp.Transparent:=True;
      sb_SetUp.Hint:=lisEFDSetup;
      IDEImages.AssignImage(sb_SetUp,'menu_environment_options');
      sb_SetUp.OnClick := @sb_SetUpClick;

end;

destructor TEmbedFormEditor.Destroy;
begin
 Inherited;
end;

Procedure TEmbedFormEditor.UpdateALLVisibleStaff;
begin
  sb_updateClick(nil);

  if FDesignPanel<> NIL then
     FDesignPanel.ExecFormCommand(CM_SHOWINGCHANGED);

 // ActivatedDesigner;
end;

Procedure TEmbedFormEditor.ActivatedDesigner;
begin
  if CT_EMBEDDED_UPDATING then exit;

  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;
  if FDesignPanel.FLoadedForm=nil then exit;

//------------------------------------------
//Fix FDesignPanel Geometry
  if FDesignPanel.FIniOK=false then
   begin
     FDesignPanel.Visible:=true;
     self.Visible:=true;

     FDesignPanel.FLoadedForm.SetBounds(FDesignPanel.FLoadedForm.Left,
                                        FDesignPanel.FLoadedForm.Top,
                                        FDesignPanel.FLoadedForm.Width+1,
                                        FDesignPanel.FLoadedForm.Height);

     FDesignPanel.FLoadedForm.SetBounds(FDesignPanel.FLoadedForm.Left,
                                        FDesignPanel.FLoadedForm.Top,
                                        FDesignPanel.FLoadedForm.Width-1,
                                        FDesignPanel.FLoadedForm.Height);

     FDesignPanel.FIniOK:=true;

     ProcEmb_FormShow(FDesignPanel.FLoadedForm);
   end;

//--------------------------------------
// Activate Embedded Form Designer
// Call ct_main.pp procedure TMainIDE.OnDesignerActivated(Sender: TObject);

  if FDesignPanel.FLoadedForm.Designer<>nil then
    TComponentEditorDesigner(FDesignPanel.FLoadedForm.Designer).IsActive:=true;

end;

procedure TEmbedFormEditor.SetupPreviewFormDesign(_Form: TCustomForm);
begin
  if (FPreviewMoving) then  Exit;

 with pnWindowScreenPreview do
  begin

  if (FDesignPanel.LoadedFormType=ctftFrame) or
     (FDesignPanel.LoadedFormType=ctftDataModule) or
     (FDesignPanel.LoadedFormType=ctftFrameWeb) or
     (FDesignPanel.LoadedFormType=ctftDataModuleWeb) then
     begin
      // TDataModules, TFrames TWebDataModules and TWebFrames
      // can not change Top or left properties
       Top := CN_BORDER_PX;
       Left:= CN_BORDER_PX;
     end else
     begin
       Top := PixelPerc(_Form.Top   , CN_PREVIEW_FACTOR);
       Left:= PixelPerc(_Form.Left  , CN_PREVIEW_FACTOR);
     end;

    Width  := PixelPerc(_Form.Width , CN_PREVIEW_FACTOR);
    Height := PixelPerc(_Form.Height, CN_PREVIEW_FACTOR);
  end;

  with pnCaptionWindowScreenPreview do
  begin
    Height := PixelPerc(Parent.Height, 15);
  end;
end;

procedure TEmbedFormEditor.InfoPanelDraw(Const aRect:TRect);
begin
  LabelX.Caption:='X: '+IntToStr(aRect.Left);
  LabelY.Caption:='Y: '+IntToStr(aRect.Top);
  LabelW.Caption:='W: '+IntToStr(aRect.Right);
  LabelH.Caption:='H: '+IntToStr(aRect.Bottom);
end;

procedure TEmbedFormEditor.DesignPanel_OnLoadForm(Sender: TObject);
begin
  //-----------------------
end;

procedure TEmbedFormEditor.DesignPanel_OnLoadedFormChangeBounds(Sender: TObject);
begin
  if FDesignPanel=Nil then Exit;
  if FDesignPanel.LoadedFormIsValid=False then Exit;

  SetupPreviewFormDesign(FDesignPanel.FLoadedForm);
end;

function TEmbedFormEditor.PixelPerc(vl, perc: single): integer;
begin
  Result := Round((vl / 100) * perc);
end;

//=== Screen Preview Mouse Events =====================

procedure TEmbedFormEditor.imgWindowScreenPreviewMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;

  SetCapture(pnWindowScreenPreview.Handle);
  FPreviewMoving := True;
  FPreviewMovingPoint.X := x;
  FPreviewMovingPoint.Y := Y;

  pnInfoPanel.Visible:=True;
end;

procedure TEmbedFormEditor.imgWindowScreenPreviewMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
 Var R:TRect;
begin
  if FPreviewMoving=false then exit;

  pnWindowScreenPreview.SetBounds(pnWindowScreenPreview.Left - (FPreviewMovingPoint.x - X),
                                  pnWindowScreenPreview.Top  - (FPreviewMovingPoint.Y - Y),
                                  pnWindowScreenPreview.Width,
                                  pnWindowScreenPreview.Height  );

  //............................................................
  R:=Rect(round(((max(pnWindowScreenPreview.Left - (FPreviewMovingPoint.X - X), 0)) / pnWindowScreenPreview.Parent.Width)  * Screen.Width ),
          round(((max(pnWindowScreenPreview.Top  - (FPreviewMovingPoint.Y - Y), 0)) / pnWindowScreenPreview.Parent.Height) * Screen.Height),
          FDesignPanel.FLoadedForm.Width,
          FDesignPanel.FLoadedForm.Height );

  InfoPanelDraw(R);
end;

procedure TEmbedFormEditor.imgWindowScreenPreviewMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FPreviewMoving=false then Exit;

  ReleaseCapture;
  FPreviewMoving := False;
  pnInfoPanel.Visible:=False;

  with pnWindowScreenPreview do
  begin
      Left := max(Left - (FPreviewMovingPoint.X - X), 0);
      Top :=  max(Top  - (FPreviewMovingPoint.Y - Y), 0);

      if FDesignPanel.LoadedForm<>nil then
      begin

        FDesignPanel.FLoadedForm.SetBounds(round((Left / Parent.Width)  * Screen.Width ),
                                           round((Top  / Parent.Height) * Screen.Height),
                                           FDesignPanel.FLoadedForm.Width,
                                           FDesignPanel.FLoadedForm.Height );

      end;
  end;

end;

//=== Bottons Mouse Events ======================================

procedure TEmbedFormEditor.sb_upClick(Sender: TObject);
 Var xMsg: TLMessage;
begin
  if FDesignPanel=nil then Exit;
  if FDesignPanel.LoadedFormIsValid=false then Exit;

  imgWindowScreenPreviewMouseDown(nil,mbLeft, [ssLeft], 0, 0);

  if Sender = sb_up     then pnWindowScreenPreview.Top  := pnWindowScreenPreview.Top  - 5;
  if Sender = sb_bottom then pnWindowScreenPreview.Top  := pnWindowScreenPreview.Top  + 5;
  if Sender = sb_left   then pnWindowScreenPreview.Left := pnWindowScreenPreview.Left - 5;
  if Sender = sb_right  then pnWindowScreenPreview.Left := pnWindowScreenPreview.Left + 5;
  if Sender = sb_zero   then
                            begin
                               pnWindowScreenPreview.Top  := CN_BORDER_PX;
                               pnWindowScreenPreview.Left := CN_BORDER_PX;
                            end;

  imgWindowScreenPreviewMouseUp( nil,mbLeft, [ssLeft], 0, 0);
end;

procedure TEmbedFormEditor.sb_updateClick(Sender: TObject);
begin
   if FDesignPanel=nil then Exit;
   if FDesignPanel.LoadedFormIsValid=false then Exit;

   imgWindowScreenPreviewMouseDown(nil,mbLeft, [ssLeft], 0, 0);

   ActivatedDesigner;

   FDesignPanel.FakeButtonsDraw(True);

   DesignPanel_OnLoadForm(nil);
   DesignPanel_OnLoadedFormChangeBounds(nil);
   FDesignPanel.FFakeWindowTitle.Repaint;
   FDesignPanel.Paint;

   imgWindowScreenPreviewMouseUp( nil,mbLeft, [ssLeft], 0, 0);
end;

procedure TEmbedFormEditor.sb_hideClick(Sender: TObject);
begin
  if pnScreenPreview.Visible then
    begin
      pnScreenPreview.Visible:=False;
      pnScreenPreviewBtns.Visible:=False;
    end else
    begin
      pnScreenPreview.Visible:=True;
      pnScreenPreviewBtns.Visible:=True;
    end;
end;

procedure TEmbedFormEditor.sb_SetUpClick(Sender: TObject);
begin
  TyphonIDE.DoOpenIDEOptions(TIDEEmbeddedDesignerSettingsFrame, '', [], []);
end;

//===========================================================================
//======================= TEmbedDesignPanel ====================================
//===========================================================================

constructor TEmbedDesignPanel.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FLoadedForm:=nil;
  FLoadedFormType:=ctftNone;
  FFakeMainMenu:=nil;
  dsParentPage:=nil;
  dsParentFormEditor:=nil;

  SetInitialBounds(0,0,screen.Width,Screen.Height);

  BevelInner := bvNone;
  BevelOuter := bvNone;

  FHasLoadedForm := False;
  FHasStoredForm := False;
  FIniOK:=false;

  FFakeWindow_MouseDown := false;

  FExecuteBeforeMessage := True;

  FMessagesHandler:=TEmbedMessageHandler.Create;

  FFakeWindow := TPanel.Create(Self);
  with FFakeWindow do
  begin
    BorderWidth := 0; // CN_BORDER_PX;
    BorderStyle := bsNone;
    BevelInner  := bvNone;
    BevelOuter  := bvRaised;
    BevelColor  := CT_EMBEDDED_TITLE_COLOR;
    OnResize    := @onFakeWindowResize;
  end;

  //--- Resizers -----------------------------------
  FResizerRight := TImage.Create(Self);
  with FResizerRight do
  begin
    tag := 100;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNS;
    OnMouseEnter:= @onResizerMouseEnter;
    OnMouseLeave:= @onResizerMouseLeave;
    OnMouseUp   := @onFakeWindowMouseUp;
    OnMouseDown := @onResizerMouseDown;
    OnMouseMove := @onResizerMouseMove;
    onPaint := @onResizerPaint;
    OnResize:= @onResizerPaint;
    visible := true;
  end;

  FResizerBottom := TImage.Create(Self);
  with FResizerBottom do
  begin
    tag := 200;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeWE;
    OnMouseEnter:= @onResizerMouseEnter;
    OnMouseLeave:= @onResizerMouseLeave;
    OnMouseUp   := @onFakeWindowMouseUp;
    OnMouseDown := @onResizerMouseDown;
    OnMouseMove := @onResizerMouseMove;
    onPaint := @onResizerPaint;
    OnResize:= @onResizerPaint;
    visible := true;
  end;

  FResizerCornerBottom := TImage.Create(Self);
  with FResizerCornerBottom do
  begin
    tag := 500;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNWSE;
    OnMouseEnter:= @onResizerMouseEnter;
    OnMouseLeave:= @onResizerMouseLeave;
    OnMouseUp   := @onFakeWindowMouseUp;
    OnMouseDown := @onResizerMouseDown;
    OnMouseMove := @onResizerMouseMove;
    onPaint := @onResizerPaint;
    OnResize:= @onResizerPaint;
    visible := true;
  end;

  FResizerCornerRight := TImage.Create(Self);
  with FResizerCornerRight do
  begin
    tag := 600;
    parent := self;
    BorderStyle := bsNone;
    BorderWidth := 0;
    BevelInner := bvNone;
    BevelOuter := bvnone;
    Cursor := crSizeNWSE;
    OnMouseEnter:= @onResizerMouseEnter;
    OnMouseLeave:= @onResizerMouseLeave;
    OnMouseUp   := @onFakeWindowMouseUp;
    OnMouseDown := @onResizerMouseDown;
    OnMouseMove := @onResizerMouseMove;
    onPaint := @onResizerPaint;
    OnResize:= @onResizerPaint;
    visible := true;
    end;

  //--- Window Title ------------------------------
  FFakeWindowTitle := TPanel.Create(FFakeWindow);
  with FFakeWindowTitle do
  begin
    Parent := FFakeWindow;
    Align := alTop;
    Height := CN_CAPTION_HEIGHT;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    ParentColor := False;
    Color := CT_EMBEDDED_TITLE_COLOR;
    OnPaint := @onFakeCaptionPaint;
  end;

  //---------------------------
    FFakeIcon := TImage.create(FFakeWindowTitle);
    with FFakeIcon do
    begin
      Parent := FFakeWindowTitle;
      BevelInner  := bvNone;
      BevelOuter  := bvNone;
      BorderStyle := bsNone;
      BorderWidth := 0;
      Height := CN_CAPTION_HEIGHT;
      Width  := Height;
      Center:=True;
      top  := 0;
      left := 0;
      Anchors := [AkTop,akLeft];
      tag := 9;
      Visible:= True;
    end;

    FFakeCaption:=TLabel.create(FFakeWindowTitle);
    with FFakeCaption do
    begin
      Parent := FFakeWindowTitle;
      BevelInner  := bvNone;
      BevelOuter  := bvNone;
      BorderStyle := bsNone;
      BorderWidth := 0;
      FFakeCaption.SetBounds(FFakeIcon.Width+4,0,
                             FFakeWindowTitle.Width-(4*CN_CAPTION_HEIGHT)-4,
                             CN_CAPTION_HEIGHT);
      AutoSize:=False;
      Alignment:=taLeftJustify;
      Layout  :=tlCenter;
      Anchors := [AkTop,akLeft,akRight];
      tag := 20;
      Font.Color := CT_EMBEDDED_TITLETEXT_COLOR;
      Visible  := True;
      Caption  :='W';
    end;

    //--- FakeBtns --------------------------------
    FFakeBtnSystem := TSpeedButton.create(FFakeWindowTitle);
    with FFakeBtnSystem do
    begin
      Parent := FFakeWindowTitle;
      BevelInner := bvNone;
      BevelOuter := bvNone;
      BorderStyle := bsNone;
      BorderWidth := 0;
      Height := CN_CAPTION_HEIGHT;
      Width  := CN_CAPTION_HEIGHT;
      top := 0;
      left := (FFakeWindowTitle.width - Width);
      Anchors := [AkTop,akRight];
      Color := CT_EMBEDDED_TITLE_COLOR;
      tag := 0;
      Transparent:=True;
      Flat:=True;
      Visible:= True;
    end;

    FFakeBtnMaximize := TSpeedButton.create(FFakeWindowTitle);
    with FFakeBtnMaximize do
    begin
      Parent := FFakeWindowTitle;
      BevelInner := bvNone;
      BevelOuter := bvNone;
      BorderStyle := bsNone;
      BorderWidth := 0;
      Height := CN_CAPTION_HEIGHT;
      Width  := CN_CAPTION_HEIGHT;
      top := 0;
      left := (FFakeBtnSystem.left - Width);
      Anchors := [AkTop,akRight];
      Caption := '';
      Color := CT_EMBEDDED_TITLE_COLOR;
      tag := 2;
      Transparent:=True;
      Flat:=True;
      Visible:= True;
    end;

    FFakeBtnMinimize := TSpeedButton.create(FFakeWindowTitle);
    with FFakeBtnMinimize do
    begin
      Parent := FFakeWindowTitle;
      BevelInner := bvNone;
      BevelOuter := bvNone;
      BorderStyle := bsNone;
      BorderWidth := 0;
      Height := CN_CAPTION_HEIGHT;
      Width  := CN_CAPTION_HEIGHT;
      top := 0;
      left := (FFakeBtnMaximize.left - Width);
      Anchors := [AkTop,akRight];
      Caption := '';
      Color := CT_EMBEDDED_TITLE_COLOR;
      tag := 4;
      Transparent:=True;
      Flat:=True;
      Visible:= true;
    end;

  FFakeWindow_MouseDown := False;
  self.OnPaint:= @ResizersDraw;

end;

destructor TEmbedDesignPanel.Destroy;
begin
  if FormIsValid(FLoadedForm) then UnLoadLoadedForm;
  FMessagesHandler.Free;
  inherited Destroy;
end;

procedure TEmbedDesignPanel.EmbeddedForm(Sender: TObject; aForm: TCustomForm);
 Var xBool:Boolean;
begin
  FLoadedFormType:=ctftNone;
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

  if Embed_IsDesignForm(aForm) then
     AddForm(aForm) else
     Exit;

  FLoadedFormType:=ctftForm;

  if (FLoadedForm is TWebForm) then
     FLoadedFormType:=ctftFormWeb     
  else if (FLoadedForm.EmbedCustomForm is TWebFrame) then
     FLoadedFormType:=ctftFrameWeb
  else if (FLoadedForm.EmbedCustomForm is TWebDataModule) then
     FLoadedFormType:=ctftDataModuleWeb
  else if (FLoadedForm.EmbedCustomForm is TFrame) then
     FLoadedFormType:=ctftFrame
  else if (FLoadedForm.EmbedCustomForm is TDataModule) then
     FLoadedFormType:=ctftDataModule
  else if SameText(FLoadedForm.ClassName,'TFrameProxyDesignerForm') then
     FLoadedFormType:=ctftFrame
  else if SameText(FLoadedForm.ClassName,'TNonControlProxyDesignerForm') then
     FLoadedFormType:=ctftDataModule;

  // Add Init Values to FakeButtons
  if (FLoadedFormType=ctftForm) then
   begin
     FLastLFormRec.aBorderStyle:= FLoadedForm.BorderStyle;
     FLastLFormRec.aBIMaximize:= (biMaximize   in FloadedForm.BorderIcons);
     FLastLFormRec.aBIMinimize:= (biMinimize   in FloadedForm.BorderIcons);
     FLastLFormRec.aBISysmenu := (biSystemMenu in FloadedForm.BorderIcons);
   end else
   begin
     FLastLFormRec.aBorderStyle:= bsNone;
     FLastLFormRec.aBIMaximize:= False;
     FLastLFormRec.aBIMinimize:= False;
     FLastLFormRec.aBISysmenu := False;
   end;

  // Load Icon to FakeIcon
  FakeIconDrawIcon;

  // Init load Icons to StageButtons
  FakeButtonsDraw(True);

  //.... TDataModules and TFrames can not change Top or left properties ......
  if dsParentFormEditor=NIL then Exit;

  xBool:=NOT ( (FLoadedFormType=ctftFrame) or
               (FLoadedFormType=ctftDataModule) or
               (FLoadedFormType=ctftFrameWeb) or
               (FLoadedFormType=ctftDataModuleWeb) );

  dsParentFormEditor.sb_zero.Enabled        :=xBool;
  dsParentFormEditor.sb_up.Enabled          :=xBool;
  dsParentFormEditor.sb_bottom.Enabled      :=xBool;
  dsParentFormEditor.sb_left.Enabled        :=xBool;
  dsParentFormEditor.sb_right.Enabled       :=xBool;
  dsParentFormEditor.pnScreenPreview.Enabled:=xBool;
end;

procedure TEmbedDesignPanel.AddForm(aForm: TCustomForm);
begin
  if aForm=nil then Exit;
  if FormIsValid(FLoadedForm) then Exit;

  if (aForm.ParentWindow = Self.Handle) or (aForm.Parent = Self) then
   begin
     DebugLN(CN_EMBEDDED_WARNING+' 201 : Try to EMBEDDED Again Form: '+aForm.ClassName);
     Exit;
   end;

  FLoadedForm:=aForm;

  ProcEmb_FormHide(FLoadedForm);

//......................................

 if dsParentPage<>nil then dsParentPage.FormEditorAdd;

 FMessagesHandler.UnHookControl;
 FMessagesHandler.HookControl(FLoadedForm);
 FMessagesHandler.OnBeforeMessage := @Self.onForm_BeforeMessage;
 FMessagesHandler.OnAfterMessage  := @Self.onForm_AfterMessage;

 FHasLoadedForm := True;
end;

procedure TEmbedDesignPanel.UnLoadLoadedForm;
begin
  if LoadedFormIsValid=False then Exit;
  try
    if FFakeWindow<>nil then FFakeWindow.Visible := false;
    if FMessagesHandler.HasHookControl then FMessagesHandler.UnHookControl;
    FLoadedForm:=nil;
  finally
    FHasLoadedForm := False;
  end;
end;


//=== Fake Buttons Functions and Mouse Events ===============================

procedure TEmbedDesignPanel.FakeIconDrawIcon;
begin
  if FFakeIcon=NIL then Exit;
  if FFakeIcon.Visible=False then Exit;

  // FFakeIcon load picture, only one time
  if FFakeIcon.Picture.Bitmap.Empty=False then Exit;

  Case FLoadedFormType of
     ctftNone          :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'state_warning');
     ctftForm          :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'form_designer');
     ctftFrame         :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'frame_designer');
     ctftDataModule    :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'datamodule_designer');
     ctftFormWeb       :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'form_designer_web');
     ctftFrameWeb      :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'frame_designer_web');
     ctftDataModuleWeb :IDEImages.AssignImage(FFakeIcon.Picture.Bitmap,'datamodule_designer_web');
  end;

end;

Procedure TEmbedDesignPanel.FakeButtonsDraw(Const ForceDraw:Boolean);
 Var xNewFormRec:TFakeFormRec;

//-----------------------------------------
 procedure xFakeButtonsFindVisibility(Const aFormRec:TFakeFormRec);
    Var xSys,xMax,xMin,eMax,eMin:Boolean;
  begin
    // Only TForms have FFakeButtons
    if (FLoadedFormType <> ctftForm) then
    Begin
      FFakeBtnSystem.Visible   := False;
      FFakeBtnMaximize.Visible := False;
      FFakeBtnMinimize.Visible := False;
      Exit;
    end;

   // Set Visible Logic
   eMax:=True;
   eMin:=True;
   xSys:= (
           (aFormRec.aBISysMenu) and
           (aFormRec.aBorderStyle <> bsNone)
            );

   xMax:= (
           (aFormRec.aBIMaximize) and   
           (xSys) and
           (aFormRec.aBorderStyle <> bsNone) and
           (aFormRec.aBorderStyle <> bsDialog) and
           (aFormRec.aBorderStyle <> bsToolWindow) and
           (aFormRec.aBorderStyle <> bsSizeToolWin)
            );

   xMin:= (
          (aFormRec.aBIMinimize) and
          (xSys) and
          (aFormRec.aBorderStyle <> bsNone) and
          (aFormRec.aBorderStyle <> bsDialog) and
          (aFormRec.aBorderStyle <> bsToolWindow) and
          (aFormRec.aBorderStyle <> bsSizeToolWin)
           );

   if (xMax=False) and (xMin=True) then
     eMax:=False;

   if (xMin=False) and (xMax=True) then
     eMin:=False;

   FFakeBtnSystem.Visible   := xSys;
   FFakeBtnMaximize.Visible := xMax or xMin;
   FFakeBtnMinimize.Visible := xMax or xMin;

   FFakeBtnMaximize.Enabled := eMax;
   FFakeBtnMinimize.Enabled := eMin;
end;
//-----------------------------------------

begin
   if FFakeWindowTitle = nil then Exit;
   if FHasLoadedForm=False then Exit;
   if FLoadedForm = nil then Exit;
   if FFakeBtnSystem = nil then exit;
   if FFakeBtnMaximize = nil then exit;
   if FFakeBtnMinimize = nil then exit;

   // Load new Form Values
   xNewFormRec.aBorderStyle:= FLoadedForm.BorderStyle;
   xNewFormRec.aBIMaximize := (biMaximize   in FloadedForm.BorderIcons);
   xNewFormRec.aBIMinimize := (biMinimize   in FloadedForm.BorderIcons);
   xNewFormRec.aBISysmenu  := (biSystemMenu in FloadedForm.BorderIcons);

  // Check if New Values are the same with Old Values
  if ForceDraw=False Then
   If (xNewFormRec.aBorderStyle= FLastLFormRec.aBorderStyle) and
      (xNewFormRec.aBIMaximize = FLastLFormRec.aBIMaximize)  and
      (xNewFormRec.aBIMinimize = FLastLFormRec.aBIMinimize)  and
      (xNewFormRec.aBISysmenu  = FLastLFormRec.aBISysmenu)  then
      Exit;

  // Load Icons
   IDEImages.AssignImage(FFakeBtnMinimize,'embed'+IntToStr(CT_EMBEDDED_TITLE_STYLE)+'_min');
   IDEImages.AssignImage(FFakeBtnMaximize,'embed'+IntToStr(CT_EMBEDDED_TITLE_STYLE)+'_max');

   Case xNewFormRec.aBorderStyle of
      bsToolWindow  : IDEImages.AssignImage(FFakeBtnSystem,'embed'+IntToStr(CT_EMBEDDED_TITLE_STYLE)+'_tw');
      bsSizeToolWin : IDEImages.AssignImage(FFakeBtnSystem,'embed'+IntToStr(CT_EMBEDDED_TITLE_STYLE)+'_stw');
   else
       IDEImages.AssignImage(FFakeBtnSystem,'embed'+IntToStr(CT_EMBEDDED_TITLE_STYLE)+'_cls');
   end;

   xFakeButtonsFindVisibility(xNewFormRec);

   // Save Form Values
   FLastLFormRec.aBorderStyle:= xNewFormRec.aBorderStyle;
   FLastLFormRec.aBIMaximize := xNewFormRec.aBIMaximize;
   FLastLFormRec.aBIMinimize := xNewFormRec.aBIMinimize;
   FLastLFormRec.aBISysmenu  := xNewFormRec.aBISysmenu;
end;

//=== Resizers Mouse Events ===========================================

procedure TEmbedDesignPanel.onResizerMouseEnter(Sender: TObject);
begin
   FResizerHighlightTag := TImage(Sender).tag;
   FResizerHighlight := True;
   if FResizerRight = nil then Exit;
   if FResizerBottom = nil then Exit;
   if FResizerCornerRight = nil then Exit;
   if FResizerCornerBottom = nil then Exit;

   ResizersUpdate;
end;

procedure TEmbedDesignPanel.onResizerMouseLeave(Sender: TObject);
begin
   FResizerHighlight := False;
   FResizerHighlightTag := 0;
   if FResizerRight = nil then Exit;
   if FResizerBottom = nil then Exit;
   if FResizerCornerRight = nil then Exit;
   if FResizerCornerBottom = nil then Exit;

   ResizersUpdate;
end;

procedure TEmbedDesignPanel.onResizerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
 Var R:TRect;
begin
  if LoadedFormIsValid=false then Exit;
  if FResizerRight = nil then Exit;
  if FResizerBottom = nil then Exit;
  if FResizerCornerRight = nil then Exit;
  if FResizerCornerBottom = nil then Exit;

  Case Tpanel(Sender).tag of
     100 :  begin   // Right
                FFakeWindow_MouseDown := True;
                FMouseArea:=100;
            end;
     200 :  begin   // Bottom
                FFakeWindow_MouseDown := True;
                FMouseArea:=200;
            end;
     500 :  begin   // Corner Bottom
                FFakeWindow_MouseDown := True;
                FMouseArea:=500;
            end;
     600 :  begin   // Corner Right
                FFakeWindow_MouseDown := True;
                FMouseArea:=600;
            end;
  end;

    GetCursorPos(FFakeWindow_MousePos);

    if dsParentFormEditor=Nil Then Exit;
    if dsParentFormEditor.pnInfoPanel=Nil Then Exit;

    if dsParentFormEditor.pnInfoPanel.Visible=False Then
       dsParentFormEditor.pnInfoPanel.Visible:=True;

    R:=Rect(FLoadedForm.Left,FLoadedForm.Top,FLoadedForm.Width,FLoadedForm.Height);
    dsParentFormEditor.InfoPanelDraw(R);

end;

procedure TEmbedDesignPanel.onResizerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
  minWidth  = 120;
  minHeight = 50;
var
  newPos, delta: TPoint;
  frmPoint : TPoint;
  R:TRect;
begin
  if LoadedFormIsValid=false then Exit;
  if dsParentFormEditor = nil then Exit;

    if FFakeWindow_MouseDown=true then
    begin
        ResizersUpdate;

        GetCursorPos(newPos);
        delta.x := newpos.x - FFakeWindow_MousePos.X;
        delta.y := newpos.y - FFakeWindow_MousePos.y;
        GetCursorPos(FFakeWindow_MousePos);
        frmPoint.x := FLoadedForm.Width + delta.x;
        frmPoint.y := FLoadedForm.Height + delta.y;

      if frmPoint.X<minWidth  then frmPoint.X:=minWidth;
      if frmPoint.Y<minHeight then frmPoint.Y:=minHeight;

      R:=Rect(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, frmPoint.Y);
      dsParentFormEditor.InfoPanelDraw(R);

       if LoadedForm<>nil then     // Resize Only if Loaded Form is Valid
       begin

          case FMouseArea of
             100: begin   // Right
                    FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, FLoadedForm.Width, frmPoint.Y);
                  end;
             200: begin  // Bottom
                    FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, FLoadedForm.Height);
                  end;
             500: begin   //Corner Bottom
                    FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, frmPoint.Y);
                  end;
             600: begin   //Corner Right
                    FLoadedForm.SetBounds(FLoadedForm.Left, FLoadedForm.Top, frmPoint.X, frmPoint.Y);
                  end;
          end;

          Self.invalidate;
       end;
  end;

end;

//=== Resizers Paint/Draw/Update Functions ===================================

procedure TEmbedDesignPanel.onResizerPaint(sender: Tobject);
 var
     xBaseColor, xHighliteColor : Tcolor;
     xImg:TImage;
begin
 if sender=Nil Then Exit;
 if NOT (sender is TImage) Then Exit;

 xImg:=TImage(sender);

 xBaseColor     := CT_EMBEDDED_RESIZERS_COLOR;
 xHighliteColor := CT_EMBEDDED_RESIZERS_HLCOLOR;

 // Fill Backround
 xImg.canvas.Brush.Style:=bsSolid;
 xImg.canvas.Brush.Color:=CT_EMBEDDED_DESIGNER_BGCOLOR;
 xImg.canvas.FillRect(0,0,xImg.Width,xImg.Height);

 // Paint Marks
 Case xImg.tag of
 100: //....Bottom ..............
     begin
        if (FResizerHighlight) then
        begin
           xImg.canvas.Pen.Color   := xHighliteColor;
           xImg.canvas.Brush.Color := xHighliteColor;
        end else
        begin
           xImg.canvas.Pen.Color   := xBaseColor;
           xImg.canvas.Brush.Color := xBaseColor;
        end;

        xImg.canvas.Rectangle(0,(xImg.Height div 2) - 1,(FFakeWindow.Width div 2) - 8, (xImg.Height div 2) + 1);
        xImg.canvas.Frame((FFakeWindow.Width div 2) - 6, 1,(FFakeWindow.Width div 2) + 3, xImg.Height);
        xImg.canvas.Rectangle((FFakeWindow.Width div 2) + 6, (xImg.Height div 2) - 1,  xImg.Width, (xImg.Height div 2) + 1);
     end;

 200: //.... Right ..............
     begin
        if (FResizerHighlight) then
        begin
          xImg.canvas.Pen.Color   := xHighliteColor;
          xImg.canvas.Brush.Color := xHighliteColor;
        end else
        begin
          xImg.canvas.Pen.Color   := xBaseColor;
          xImg.canvas.Brush.Color := xBaseColor;
        end;

        xImg.canvas.rectangle((xImg.Width div 2) - 1,  0,  (xImg.Width div 2) + 1, (FFakeWindow.Height div 2) - 8);
        xImg.canvas.Frame(1, (FFakeWindow.Height div 2) - 5, xImg.Width, (FFakeWindow.Height div 2) + 4);
        xImg.canvas.rectangle((xImg.Width div 2) - 1, (FFakeWindow.Height div 2) + 6,  (xImg.Width div 2) + 1, xImg.Height);
    end;

 500: //.... Corner Bottom .............
     begin
          if (FResizerHighlight) then
          begin
            FResizerCornerBottom.canvas.Pen.Color  := xHighliteColor;
            FResizerCornerBottom.canvas.Brush.Color:= xHighliteColor;
          end else
          begin
            FResizerCornerBottom.canvas.Pen.Color  := xBaseColor;
            FResizerCornerBottom.canvas.Brush.Color:= xBaseColor;
          end;

          FResizerCornerBottom.canvas.rectangle((FResizerCornerBottom.Width div 2) - 1,2,(FResizerCornerBottom.Width div 2) + 1,FResizerCornerBottom.Height - 2);
     end;

 600: //.... Corner Right .............
     begin
          if (FResizerHighlight) then
          begin
            FResizerCornerRight.canvas.Pen.Color  := xHighliteColor;
            FResizerCornerRight.canvas.Brush.Color:= xHighliteColor;
          end else
          begin
            FResizerCornerRight.canvas.Pen.Color  := xBaseColor;
            FResizerCornerRight.canvas.Brush.Color:= xBaseColor;
          end;

          FResizerCornerRight.canvas.rectangle(2,(FResizerCornerRight.Height div 2) - 1,FResizerCornerRight.Width  - 12,(FResizerCornerRight.Height div 2) + 1);
          FResizerCornerRight.canvas.Frame(FResizerCornerRight.Width - 10,FResizerCornerRight.Height - 9, FResizerCornerRight.Width-1, FResizerCornerRight.Height);
     end;

 end; //Case End

end;

procedure TEmbedDesignPanel.ResizersDraw(Sender: TObject);
begin
   if FLoadedForm = nil then exit;
   if FFakeWindow = nil then Exit;
   if not FFakeWindow.Visible then Exit;

   FFakeWindow.BevelColor:= CT_EMBEDDED_TITLE_COLOR;

   IF FResizerRight <> nil then
   begin
      FResizerRight.SetBounds(FFakeWindow.left + 1 , (FFakeWindow.top + FFakeWindow.Height) + 5 , FFakeWindow.Width - 17,10);
   end;
   IF FResizerBottom <> nil then
   begin
      FResizerBottom.SetBounds( FFakeWindow.left + FFakeWindow.width + 5,FFakeWindow.top + 1, 10, FFakeWindow.Height - 17 );
   end;
   IF FResizerCornerRight <> nil then
   begin
     FResizerCornerRight.SetBounds(FResizerRight.Left + FResizerRight.Width + 2 , FResizerRight.top , 30,10);
   end;
   IF FResizerCornerBottom <> nil then
   begin
      FResizerCornerBottom.SetBounds( FResizerBottom.left, FResizerBottom.top + FResizerBottom.Height + 2, 10, 20);
   end;

end;

procedure TEmbedDesignPanel.ResizersUpdate;
begin
  onResizerPaint(FResizerCornerRight);
  onResizerPaint(FResizerCornerBottom);
  onResizerPaint(FResizerBottom);
  onResizerPaint(FResizerRight);
end;

//=======================================================================

function TEmbedDesignPanel.FormIsValid(aForm: TCustomForm): Boolean;
begin
  Result := (Assigned(aForm)) and (aForm <> Nil);
end;

function TEmbedDesignPanel.LoadedFormIsValid: Boolean;
begin
  Result := FormIsValid(FLoadedForm);
end;

function TEmbedDesignPanel.FormZeroOrigin(aForm: TCustomForm): TPoint;

//------------------------------------------
  procedure xFindFormBorder;
    var
        w: TWinControl;
        rc1, rc2: TRect;
   begin
    if CN_REMOVE_FORM_BORDER=False then exit;

    try
      w := TWinControl.Create(Self);
      w.Left := 0;
      w.Top  := 0;
      w.Parent := aForm;

      GetWindowRect(w.Handle, rc1);
      GetWindowRect(aForm.Handle, rc2);

      Result.x := Max(rc1.Left - rc2.Left, 0);
      Result.y := Max(rc1.Top  - rc2.Top,  0);
    finally
      w.free;
    end;

   end;

//------------------------------------------
  procedure xFindCompSizes;
    var i: Integer;
   begin                       
     FFormMainMenuHeight:=0;
     if CN_MUST_FIND_MAINMENU_HEIGHT=False then Exit;

     for i := 0 to aForm.ComponentCount - 1 do
      begin
        if aForm.Components[i] is TMainMenu then
         if TMainMenu(aForm.Components[i]).Items.Count>0 then           // Don't add Height of TmainMenu with No Items
          begin
            FFormMainMenuHeight := LCLIntf.GetSystemMetrics(SM_CYMENU); //789: Fix for TMainMenu + TStatusBar Problem
            Result.y := Result.y - FFormMainMenuHeight;
            Break;
            Exit;
          end;
      end;
   end;

//-----------------------------------------
begin
Result := Point(0,0);

if FormIsValid(aForm)=false then exit;

if FFakeWindow_MouseDown then
begin
 result:=FZeroOriginPoint;
 EXIT;
end;

xFindFormBorder;
xFindCompSizes;

FZeroOriginPoint:=Result;
end;

Function TEmbedDesignPanel.CheckForFakeMainMenu(aForm: TCustomForm):Boolean;
    var i: Integer;
        xMainMenu:TMainMenu;
begin

Result:=False;

if FormIsValid(aForm)=False then Exit;
if Embed_IsDesignForm(aform)=false then Exit;
if CN_USE_FAKE_MAINMENU=False then Exit;
if FFakeWindow_MouseDown then EXIT;

xMainMenu:=nil;

//..... Find Form MainMenu ...
for i := 0 to aForm.ComponentCount - 1 do
  begin
    if aForm.Components[i] is TMainMenu then
      if TMainMenu(aForm.Components[i]).Items.Count>0 then
       begin
        xMainMenu:=TMainMenu(aForm.Components[i]);
       end;
  end;

Result:=(xMainMenu<>nil);

//............................
if xMainMenu=nil then
begin
  if FFakeMainMenu<>nil then
    begin
      FFakeMainMenu.Free;
      FFakeWindowTitle.Height:=CN_CAPTION_HEIGHT;
    end;
    FFakeMainMenu:=nil;
end else
begin
  if FFakeWindow=nil then Exit;

  if FFakeMainMenu=nil then
  begin
     FFakeMainMenu:=TTEmbedFakeMainMenu.Create(FFakeWindowTitle);
     FFakeMainMenu.Parent:=FFakeWindowTitle;
     FFakeMainMenu.MainMenuAssign(xMainMenu);
     FFakeWindowTitle.Height:=CN_CAPTION_HEIGHT+FFakeMainMenu.Height;
  end;
end;
end;

procedure TEmbedDesignPanel.UpdateFakeMainMenu;
begin
if CN_USE_FAKE_MAINMENU=False then Exit;
if FFakeMainMenu=nil then Exit;

FFakeMainMenu.MainMenuUpdate;
end;

// Execute Commands to Loaded Form like:
// LM_ACTIVATE, LM_SETFOCUS, LM_MOVE, LM_SIZE, CM_SHOWINGCHANGED, etc
procedure TEmbedDesignPanel.ExecFormCommand(Const aCommand: UINT);
 var xMsg: TLMessage;
begin
    xMsg.msg:=aCommand;
    onForm_AfterMessage(FMessagesHandler,FLoadedForm,xMsg);
end;

//=============================== Before LoadedForm Message =====================
procedure TEmbedDesignPanel.onForm_BeforeMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);

//..................................................................
  procedure _AdjustSelfPosition(aForm: TCustomForm);
    Var pRect,R:TRect;

  begin

    if aForm.ParentWindow <> Self.Handle then
    begin
      Self.SetBounds(0,0,Self.Width,Self.Height);
    end;

    GetWindowRect(Self.Handle, pRect);

    R:=Rect(-(pRect.Left - Self.Left),
                   -(pRect.Top  - Self.Top),
                     Self.Width,
                     Self.Height  );

    if (Self.Left  <>R.Left)  or
       (Self.Top   <>R.Top)   or
       (Self.Width <>R.Right) or
       (Self.Height<>R.Bottom) then
       FFakeWindow.SetBounds(R.Left, R.Top, R.Right, R.Bottom);
  end;
//..................................................................
begin
if NOT FExecuteBeforeMessage     then Exit;
if NOT (Sender=FMessagesHandler) then Exit;
if LoadedFormIsValid=false       then Exit;
if TCustomForm(SenderForm)<>FLoadedForm then Exit;

//debugln('TEmbedDesignPanel.onForm_BeforeMessage : '+GetMessageName(Msg.msg)); //--- ct7777 -----

//--------------------------------------------------------

  if NOT (csDestroying in FLoadedForm.ComponentState) then
    if ((CN_FORM_DESIGNER_PARENT_TYPE=ctptParentWindow) and (FLoadedForm.ParentWindow <> Self.Handle)) or
       ((CN_FORM_DESIGNER_PARENT_TYPE=ctptParent) and (FLoadedForm.Parent <> Self)) or
       ( CN_FORM_DESIGNER_PARENT_TYPE=ctptDock) or
       ( CN_FORM_DESIGNER_PARENT_TYPE=ctptNone) then
         begin
            FExecuteBeforeMessage := False;
            Embed_SetFormParent(FLoadedForm, Self);
            FExecuteBeforeMessage := True;
            if FHasStoredForm then
            begin
              EmbStoreForms.FormUnStore(FLoadedForm);
              FHasStoredForm:=False;
            end;
            if Assigned(OnFormLoad) then OnFormLoad(Self);
            if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
         end;
 //--------------------------------------------------------
end;

//=============================== After LoadedForm Message =====================
procedure TEmbedDesignPanel.onForm_AfterMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
 var pRect: TRect;

//..................................................................
  procedure _UpdateFakeWindow(aForm: TCustomForm);
   var zoPoint:TPoint;
       R:TRect;
       B:Boolean;
  begin
      FFakeWindow.Visible := ((aForm <> Nil) and (Assigned(aForm)));
      B:=False;

      if FFakeWindow.Visible then
      begin
        zoPoint:=FormZeroOrigin(aForm);

        R:=Rect(
                ((aForm.Left)  - (FFakeWindowTitle.Left)) + zoPoint.x,
                ((aForm.Top )  - (FFakeWindowTitle.Top + FFakeWindowTitle.Height)) + zoPoint.y,
                  aForm.Width  + (FFakeWindowTitle.Left * 2),
                  aForm.Height + (FFakeWindowTitle.Top + FFakeWindowTitle.Height) + FFakeWindowTitle.Top
                );

        if (FFakeWindow.Left  <>R.Left)   or
           (FFakeWindow.Top   <>R.Top)    or
           (FFakeWindow.Width <>R.Right)  or
           (FFakeWindow.Height<>R.Bottom) then
           Begin
            B:=True;
            FFakeWindow.SetBounds(R.Left, R.Top, R.Right, R.Bottom + FFormMainMenuHeight); //789: Fix for TMainMenu + TStatusBar Problem
           end;
      end;

      if FFakeWindow_MouseDown then exit;
      if B=False then Exit;

      onFakeWindowResize(FFakeWindow);
  end;

//..................................................................
  procedure _UpdateLoadedFormBorder(aForm: TCustomForm);
  var
    xRegion: TRegion;
  begin
    if CN_REMOVE_FORM_BORDER=false then Exit;

    with aForm, FormZeroOrigin(aForm) do
    begin
      xRegion := TRegion.Create;
       try
         xRegion.AddRectangle(  x,
                                y,
                                x + Width,
                                y + Height+FFormMainMenuHeight );  //789: Fix for TMainMenu + TStatusBar Problem

         aForm.SetShape(xRegion);
       finally
         xRegion.Free;
      end;

    end;
  end;

//..................................................................
  procedure _UpdateFormDesigner(aForm: TCustomForm);
  begin
    with TCustomForm(aForm) do UpdateControlState;
  end;

//..................................................................
  procedure _UpdateSelf;
    var R:TRect;
   begin

     if FFakeWindow.Visible then
         R:=Rect((-FFakeWindow.Left) + 5,(-FFakeWindow.Top)  + 5, Screen.Width, Screen.Height) else
         R:=Rect((-FLoadedForm.Left) + 5,(-FLoadedForm.Top)  + 5, Screen.Width, Screen.Height);

     if (Self.Left  <>R.Left)  or
        (Self.Top   <>R.Top)   or
        (Self.Width <>R.Right) or
        (Self.Height<>R.Bottom) then
         Self.SetBounds(R.Left, R.Top, R.Right, R.Bottom);
   end;

//..................................................................

begin
if FHasLoadedForm=false then exit;
if NOT (Sender=FMessagesHandler) then Exit;
if LoadedFormIsValid=false then Exit;
if TCustomForm(SenderForm)<>FLoadedForm  then Exit;

if (csDestroying in FLoadedForm.ComponentState) then Exit;
if (csFreeNotification in FLoadedForm.ComponentState) then Exit;
if (csLoading in FLoadedForm.ComponentState) then Exit;
if (csUpdating in FLoadedForm.ComponentState) then Exit;
if (csReading in FLoadedForm.ComponentState) then Exit;
if (csWriting in FLoadedForm.ComponentState) then Exit;

//--------------------------------
case Msg.msg of
LM_SHOWWINDOW:
              begin
              if TLMShowWindow(Msg).Show then
                begin

                  if CN_USE_FAKE_MAINMENU then
                    if CheckForFakeMainMenu(FLoadedForm) then
                       UpdateFakeMainMenu;

                  _UpdateFakeWindow(FLoadedForm);
                  _UpdateLoadedFormBorder(FLoadedForm);
                  _UpdateSelf;

                end;
              end;

LM_ACTIVATE,
LM_SETFOCUS,
LM_MOVE,
LM_SIZE,
CM_SHOWINGCHANGED:
             begin
                 _UpdateFakeWindow(FLoadedForm);
                 _UpdateLoadedFormBorder(FLoadedForm);
                 _UpdateSelf;
                 _UpdateFormDesigner(FLoadedForm);
                 if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                 if FFakeWindowTitle<>nil then FFakeWindowTitle.Repaint;

             end;

CM_TEXTCHANGED:    //Change Stored Form/Module/Frame Caption/Name
             begin
                if FFakeWindowTitle<>nil then
                   FFakeWindowTitle.Repaint;
             end;

CM_MENUCHANGED,
CM_CONTROLLISTCHANGE:   // for Fake MainMemu added
             begin

               if CN_USE_FAKE_MAINMENU=false then
               begin   // for Original MainMemu added

                    if dsParentFormEditor<>NIL then
                       dsParentFormEditor.sb_updateClick(nil);

               end else
               begin   // for Fake MainMemu added

                    if FFakeMainMenu=nil then   // Used Only for Init Greate FakeMainMenu
                     if CheckForFakeMainMenu(FLoadedForm) then
                     begin
                       UpdateFakeMainMenu;
                       _UpdateFakeWindow(FLoadedForm);
                       _UpdateLoadedFormBorder(FLoadedForm);
                       _UpdateSelf;
                       if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                    end;

               end;
             end;

LM_ERASEBKGND,
LM_PAINT:
             begin
                if FLoadedForm = nil then Exit;

                if Msg.msg=LM_PAINT then
                   FakeButtonsDraw(False);

               //--- for Fake MainMemu Deleted  -----------------------
                if CN_USE_FAKE_MAINMENU=false then exit;
                if FFakeMainMenu=nil then exit;             // Useded Only for Init Greate FakeMainMenu

                if CheckForFakeMainMenu(FLoadedForm) then   // FakeMainMenu Exists so Update Items
                 begin
                   UpdateFakeMainMenu;
                 end else                                   // FakeMainMenu Deleted so Update DesignPanel
                 begin
                   _UpdateFakeWindow(FLoadedForm);
                   _UpdateLoadedFormBorder(FLoadedForm);
                   _UpdateSelf;
                   if Assigned(OnLoadedFormChangeBounds) then  OnLoadedFormChangeBounds(Self);
                 end;
               //-----------------------------------------------------
            end;
end;

end;

procedure TEmbedDesignPanel.onFakeWindowMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FFakeWindow_MouseDown=False then Exit;

  FFakeWindow_MouseDown := False;
  dsParentFormEditor.pnInfoPanel.Visible:=False;
  if FResizerRight = nil then Exit;
  if FResizerBottom = nil then Exit;
  if FResizerCornerRight = nil then Exit;
  if FResizerCornerBottom = nil then Exit;

  FResizerRight.Invalidate;
  FResizerBottom.Invalidate;
  FResizerCornerRight.Invalidate;
  FResizerCornerBottom.Invalidate;

end;

procedure TEmbedDesignPanel.onFakeWindowResize(Sender: TObject);
var
  fk_rgn, fm_rgn, Region, Region2 : HRGN;
  fk_rc, fm_rc: TRect;
  xRegion : TRegion;
begin

  fk_rc.Left   := 0;
  fk_rc.Top    := 0;
  fk_rc.Right  := FFakeWindow.Width;
  fk_rc.Bottom := FFakeWindow.Height;

  with fm_rc do
  begin
    Left   := FFakeWindowTitle.Left;
    Top    := FFakeWindowTitle.Top + FFakeWindowTitle.Height;
    Right  := Left + FFakeWindowTitle.Width;
    Bottom := FFakeWindow.Height - (FFakeWindowTitle.Top);
  end;

  fk_rgn := CreateRectRgn(fk_rc.Left, fk_rc.Top, fk_rc.Right, fk_rc.Bottom);
  fm_rgn := CreateRectRgn(fm_rc.Left, fm_rc.Top, fm_rc.Right, fm_rc.Bottom);

  CombineRgn(fk_rgn, fk_rgn, fm_rgn, RGN_DIFF);

  //........................................
  GetRgnBox(fk_rgn,PRect(@fk_rc));

  xRegion := TRegion.Create;
       try
         xRegion.AddRectangle(
                                fk_rc.Left,
                                fk_rc.Top,
                                fk_rc.Right,
                                fk_rc.Bottom);

         FFakeWindow.SetShape(xRegion);
       finally
         xRegion.Free;
      end;

end;

procedure TEmbedDesignPanel.onFakeCaptionPaint(Sender: TObject);
 Var xss : string;
begin
  if FHasLoadedForm=false then exit;
  if LoadedFormIsValid=false then Exit;
  if FHasLoadedForm=False then Exit;
  if FLoadedForm = nil then Exit;
  if FFakeCaption = nil then Exit;

  if FFakeCaption.Caption=FLoadedForm.Caption Then Exit;

  xss:='Error: NO Form';
  if FLoadedForm<>Nil then xss:=FLoadedForm.Caption;
  if xss='' then xss:='NO Form Caption';

  FFakeCaption.Caption:=xss;
end;

procedure TEmbedDesignPanel.InitializeWnd;
begin
  inherited InitializeWnd;

  with FFakeWindow do
  begin
    Parent := Self;
    ParentColor := False;
    Color := clForm;
  end;

end;

//===========================================================================
//======================== TEmbedMessageHandler =============================
//===========================================================================
constructor TEmbedMessageHandler.Create;
begin
  inherited Create;
  SetDefaulOptions;
  FHookControl    := nil;
  OnBeforeMessage := nil;
  OnAfterMessage  := nil;
end;

destructor TEmbedMessageHandler.Destroy;
begin
  UnHookControl;
  inherited Destroy;
end;

Procedure TEmbedMessageHandler.SetDefaulOptions;
begin
 FOptions:=[ehoSendBefore,ehoSendAfter,ehoSendToForm];
end;

procedure TEmbedMessageHandler.HookControl(AControl: TControl);
begin
  SetDefaulOptions;

  if AControl=nil then exit;
  FHookControl := AControl;
  InheritedWndMethod :=  FHookControl.WindowProc; //Save Control WindowProc
  FHookControl.WindowProc := @SelfWndMethod;      //Change Control WindowProc
end;

procedure TEmbedMessageHandler.UnHookControl;
begin
  OnBeforeMessage := nil;
  OnAfterMessage  := nil;
  if NOT HasHookControl then Exit;
  FHookControl.WindowProc:=InheritedWndMethod;   //Restore Control WindowProc
  FHookControl:=nil;
  InheritedWndMethod:=nil;

  SetDefaulOptions;
end;

function TEmbedMessageHandler.HasHookControl: Boolean;
begin
  Result := (Assigned(FHookControl)) and (FHookControl <> Nil);
end;

procedure TEmbedMessageHandler.SelfWndMethod(var TheMessage: TLMessage);
begin
  if CT_EMBEDDED_UPDATING then Exit;
  if (not HasHookControl) then Exit;

 // debugln('TEmbedMessageHandler.SelfWndMethod : '+GetMessageName(TheMessage.msg)); //--- ct7777 -----

  case TheMessage.msg of
     LM_SHOWWINDOW,
     LM_ACTIVATE,
     LM_SETFOCUS,
     LM_MOVE,
     LM_SIZE,
     LM_ERASEBKGND,
     LM_PAINT,
     CM_SHOWINGCHANGED,    // Must for MacOS
     CM_MENUCHANGED,
     CM_TEXTCHANGED,
     CM_CONTROLLISTCHANGE
                :begin
                  if (ehoSendBefore in Foptions) and Assigned(OnBeforeMessage)    then OnBeforeMessage(Self, FHookControl, TheMessage);
                  if (ehoSendToForm in Foptions) and Assigned(InheritedWndMethod) then InheritedWndMethod(TheMessage);
                  if (ehoSendAfter in Foptions)  and Assigned(OnAfterMessage)     then OnAfterMessage(Self, FHookControl, TheMessage);
                 end;
  else
     if (ehoSendToForm in Foptions) and Assigned(InheritedWndMethod) then InheritedWndMethod(TheMessage);
  end;

end;

//===========================================================================
//======================= TTEmbedFakeMainMenu ===============================
//===========================================================================
constructor TTEmbedFakeMainMenu.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FMainMenu:=nil;
  Color:=clMenuBar;
  EdgeBorders:=[ebTop,ebBottom];
  ShowCaptions:=true;
  Align:=alBottom;
  List:=True;
end;

destructor TTEmbedFakeMainMenu.Destroy;
begin
  MainMenuClear;
  inherited Destroy;
end;

Procedure TTEmbedFakeMainMenu.MainMenuAssign(aMainMenu:TMainMenu);
begin
 FMainMenu:=aMainMenu;
 if FMainMenu=nil then exit;

 self.Images:=FMainMenu.Images;
end;

Procedure TTEmbedFakeMainMenu.MainMenuClear;
  var i:Integer;
begin

 for i := ButtonCount - 1 downto 0 do
   begin
     TToolButton(Buttons[i]).MenuItem:=nil;
     TToolButton(Buttons[i]).Free;
   end;

 ButtonList.Clear;
end;

Procedure TTEmbedFakeMainMenu.MainMenuUpdate;
  var i:Integer;
      xMenu    :TMenuItem;
      xButton  :TToolButton;

  Procedure _SetMenuItem(const val:integer);
  begin
      xMenu:=FMainMenu.Items[i];

      if xMenu<>nil then
       if xMenu.Visible then
       begin
        xButton:=TToolButton.Create(self);
        xButton.Parent:=self;
        xButton.Name:=xMenu.Name;
        xButton.Caption :=xMenu.Caption;
        xButton.MenuItem:=xMenu;
       end;
    end;

begin
if FMainMenu=nil then exit;

MainMenuClear;

if FMainMenu.IsRightToLeft=false then
begin
  for i:=FMainMenu.Items.Count-1 Downto 0 do _SetMenuItem(i);
end else
begin
  for i:=0 to FMainMenu.Items.Count-1     do _SetMenuItem(i);
end;

end;

//===========================================================================
//========================== TEmbedTabSheet =================================
//===========================================================================

constructor TEmbedTabSheet.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FEditPages:=nil;
  FEditPageForSource:=nil;
  FEditPageForDesigner:=nil;
  FFormEditor:=nil;
  FSynEditor:=nil;
end;

destructor TEmbedTabSheet.Destroy;
begin
  inherited Destroy;
end;

procedure TEmbedTabSheet.EditPages_Create;
begin
  if FEditPages<>nil then exit;

  CT_EMBEDDED_UPDATING:=True;

  //-----------------
  FEditPages:= TPageControl.Create(self);
  FEditPages.Parent:=self;
  FEditPages.Images := IDEImages.Images_16;
  FEditPages.Align:=alClient;//alBottom;
  FEditPages.TabPosition:=tpBottom;
  FEditPages.ShowTabs:=CT_USE_EMBEDDED_DESIGNER;

  //--- Page for Source code SynEdit ----
  FEditPageForSource:=FEditPages.AddTabSheet;
  FEditPageForSource.Name:='EditPageForSource1';
  FEditPageForSource.Caption:='Source Editor';
  FEditPageForSource.TabVisible:=true;
  FEditPageForSource.ImageIndex:= IDEImages.GetImageIndex('menu_view_source_editor');

  //--- Page for FormDesigner -----------
  FEditPageForDesigner:=FEditPages.AddTabSheet;
  FEditPageForDesigner.Name:='EditPageForDesigner1';
  FEditPageForDesigner.Caption:='Form Designer';
  FEditPageForDesigner.TabVisible:=false;     
  FEditPageForDesigner.ImageIndex:= IDEImages.GetImageIndex('menu_view_form_editor');

  FEditPages.OnChange:=@DoOnTabControlChange;

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.EditPages_Free;
begin
  if FEditPages=nil then exit;

  CT_EMBEDDED_UPDATING:=True;

  FormEditorDelete;
  FEditPages.Free;

  FEditPages:=nil;
  FEditPageForSource:=nil;
  FEditPageForDesigner:=nil;

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.DoOnTabControlChange(Sender: TObject);
begin
  if CT_EMBEDDED_UPDATING then exit;
  if FEditPages=nil then exit;

  if FEditPages.ActivePage=FEditPageForDesigner then
   if FFormEditor<>nil then
      FFormEditor.UpdateALLVisibleStaff;

  if FEditPages.ActivePage=FEditPageForSource then
   if FSynEditor<>nil then    
     if FSynEditor.Focused=False then
        FSynEditor.SetFocus;

end;

procedure TEmbedTabSheet.ShowSourceTab;
begin
 if CT_EMBEDDED_UPDATING then exit;
 if FEditPages=Nil then Exit;
 if FEditPageForSource=Nil then Exit;

 if (FEditPages.ActivePage<>FEditPageForSource) then
     FEditPages.ActivePage:=FEditPageForSource;

 if FSynEditor<>nil then
   if FSynEditor.Focused=False then
      FSynEditor.SetFocus;

end;

procedure TEmbedTabSheet.ShowFormEditorTab;
 var xform:TCustomForm;
begin
 if CT_EMBEDDED_UPDATING then exit;

 if FEditPages=nil then exit;

//--- Check FFormEditor ------------------
 xform:=nil;

  if FFormEditor<>nil then
    if FFormEditor.FDesignPanel<>nil then
       if FFormEditor.FDesignPanel.FHasLoadedForm=false then
       begin
         xform:=FFormEditor.FDesignPanel.FLoadedForm;   // Get LoadedForm
         FormEditorDelete;                              // Delete OLD FormEditor

         if xform<>nil then
         begin

           if (CN_FORM_DESIGNER_PARENT_TYPE=ctptParentWindow) then
            if xForm.ParentWindow<>0 then
               xForm.ParentWindow:=0;                   // Reset LoadedForm ParentWindow

           if xForm.Parent<>Nil Then
              xForm.Parent:=nil;                        // Reset LoadedForm Parent

           ProcEmb_FormHide(xform);                     // Hide  LoadedForm

           EmbeddedForm(nil,xform,                      // Embedded LoadedForm
                            self.LoadedFormSourceWinNum,
                            self.LoadedFormEmbedTabSheetNum,
                            self.LoadedFormClassName,
                            self.LoadedFormFileName);
         end;
      end;

//--------------------------------------

  if FEditPages=nil then exit;
  if FEditPageForDesigner.TabVisible then               // Only if Tab is visible for "Clone" Forms
   if (FEditPages.ActivePage<>FEditPageForDesigner) then
    begin

     FEditPages.ActivePage:=FEditPageForDesigner;
     if FFormEditor<>nil then
        FFormEditor.UpdateALLVisibleStaff;

    end;
end;

Procedure TEmbedTabSheet.UpdateEmbedFormDsgSettings;
begin
  if NOT Assigned(FFormEditor) then exit;

  CT_EMBEDDED_UPDATING:=True;

  FFormEditor.RootPanel.ParentColor:=false;
  FFormEditor.RootPanel.Color            := CT_EMBEDDED_DESIGNER_BGCOLOR;

  FFormEditor.pnDesignBox.ParentColor:=false;
  FFormEditor.pnDesignBox.Color          := CT_EMBEDDED_DESIGNER_BGCOLOR;

  FFormEditor.pnInfoPanel.Color          := CT_EMBEDDED_INFOPANEL_COLOR;
  FFormEditor.pnInfoPanel.Font.Color     := CT_EMBEDDED_INFOPANEL_COLOR_TEXT;

  FFormEditor.pnScreenPreview.Color      := CT_EMBEDDED_SRCEENPREVIEW_BGCOLOR;
  FFormEditor.pnScreenPreview.Visible    := CT_EMBEDDED_USE_SRCEENPREVIEW;

  FFormEditor.pnScreenPreviewBtns.Color  := CT_EMBEDDED_DESIGNER_BGCOLOR;
  FFormEditor.pnScreenPreviewBtns.Visible:= CT_EMBEDDED_USE_SRCEENPREVIEW;

  if NOT Assigned(FFormEditor.FDesignPanel) then exit;

  if FFormEditor.FDesignPanel.FFakeWindowTitle<>NIL then
     FFormEditor.FDesignPanel.FFakeWindowTitle.Color:=CT_EMBEDDED_TITLE_COLOR;

  if FFormEditor.FDesignPanel.FFakeCaption<>NIL then
     FFormEditor.FDesignPanel.FFakeCaption.Font.Color:=CT_EMBEDDED_TITLETEXT_COLOR;

  FFormEditor.FDesignPanel.FakeButtonsDraw(True);

  CT_EMBEDDED_UPDATING:=False;
end;

procedure TEmbedTabSheet.FormEditorsINIT;
begin
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

  CT_EMBEDDED_UPDATING:=True;
  if FFormEditor<>nil then FFormEditor.Free;

  //........... FFormEditor .........................
  FFormEditor:=TEmbedFormEditor.Create(FEditPageForDesigner);
  FFormEditor.Parent:=FEditPageForDesigner;
  FFormEditor.Visible:=false;
  FFormEditor.Align:=alClient;

  //........... Design Panel .......................
  FFormEditor.FDesignPanel := TEmbedDesignPanel.Create(FFormEditor.pnDesignBox);
  FFormEditor.FDesignPanel.Parent := FFormEditor.pnDesignBox;
  FFormEditor.FDesignPanel.Visible:=false;
  FFormEditor.FDesignPanel.dsParentPage:=self;
  FFormEditor.FDesignPanel.dsParentFormEditor:=FFormEditor;
  FFormEditor.FDesignPanel.OnFormLoad := @FFormEditor.DesignPanel_OnLoadForm;
  FFormEditor.FDesignPanel.OnLoadedFormChangeBounds := @FFormEditor.DesignPanel_OnLoadedFormChangeBounds;

  UpdateEmbedFormDsgSettings;

  FEditPageForDesigner.TabVisible:=true;

  CT_EMBEDDED_UPDATING:=False;
end;

Procedure TEmbedTabSheet.FormEditorDELETE;
begin
if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;

 if FFormEditor<>nil then FFormEditor.free;
 FFormEditor:=nil;

 if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=false;
end;

Procedure TEmbedTabSheet.FormEditorAdd;
begin
 if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;
 if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=true;
end;

Function TEmbedTabSheet.GetLoadedForm:TCustomForm;
begin
 Result:=nil;

 if FFormEditor<>nil then
   if FFormEditor.FDesignPanel<>nil then
     if FFormEditor.FDesignPanel.LoadedForm<>nil then
        Result:=FFormEditor.FDesignPanel.LoadedForm;
end;


// ======= Note-100 SOS SOS SOS ===============================================
// Some Forms/Frames/DataModules have multi-level Inherited
// Like: Tform3 => Tform2 => Tform1 => Tform.
// In these case the IDE load the Base level Tform and
// then load in to the SAME TCustomForm the next Inherited Levels
// Tform1 => Tform2 => Tform3 => etc
// The Same with Frames and DataModules.
//
// Sample1: typhon\components\pl_FirebirdDB\source\design\ibsqleditor.pas
// Sample2: typhon\components\pl_FirebirdDB\source\design\ibmodifysqleditor.pas
//==============================================================================
Function TEmbedTabSheet.EmbeddedForm(Sender: TObject; aForm: TCustomForm; Const aWin,aETS:Integer; Const aFCN,aFile:String):Boolean;
//................................................
// Embedded Form/Frame/DataModule for the First Time.
  Function xEMBEDDED_FORM:Integer;
  begin
      Result:=0;
      FormEditorsINIT;

      if FFormEditor=nil then exit;
      if FFormEditor.FDesignPanel=nil then exit;

      CT_EMBEDDED_UPDATING:=True;
      FFormEditor.FDesignPanel.EmbeddedForm(Sender,aForm);

      if FEditPageForDesigner<> nil then FEditPageForDesigner.TabVisible:=true;

      FFormEditor.FDesignPanel.IniOK:=false;
      FFormEditor.ActivatedDesigner;

     CT_EMBEDDED_UPDATING:=False;
     Result:=1;
  end;
//.................................................
// ReEmbedded Form/Frame/DataModule for multi-level Inherited, look above Note-100
  Function xRE_EMBEDDED_FORM:Integer;
  begin
     Result:=0;
     if aForm=NIL then Exit;
     if FFormEditor=Nil then Exit;
     if FFormEditor.FDesignPanel=Nil then Exit;
     if FFormEditor.FDesignPanel.Visible=True then Exit; // SOS SOS SOS

     xEMBEDDED_FORM;

     if FFormEditor<>Nil then
      if FFormEditor.FDesignPanel<>Nil then
         FFormEditor.FDesignPanel.Visible:=False;

     if aForm=nil then
         DebugLn(EMBEDDED_FORM_DESIGNER,'EMBEDDED ( RE_EMBEDDED_FORM) /SWin:'+ IntToStr(aWin)+' /ETS:'+IntToStr(aETS)+' /FCN:NIL') else
         DebugLn(EMBEDDED_FORM_DESIGNER,'EMBEDDED ( RE_EMBEDDED_FORM) /SWin:'+ IntToStr(aWin)+
                                        ' /ETS:' +IntToStr(aETS)+
                                        ' /FCN:' +aFCN+
                                        ' /File:'+aFile);

     Result:=1;
  end;
//.................................................
begin
  Result:=False;
  if FEditPages=nil then exit;

  if FFormEditor=nil then
   begin  // Embedded Form/Frame/DataModule for the First Time.
     Result:=(xEMBEDDED_FORM=1);

     if aForm=nil then
         DebugLn(EMBEDDED_FORM_DESIGNER,'EMBEDDED ( FT_EMBEDDED_FORM) /SWin:'+ IntToStr(aWin)+' /ETS:'+IntToStr(aETS)+' /FCN:NIL') else
         DebugLn(EMBEDDED_FORM_DESIGNER,'EMBEDDED ( FT_EMBEDDED_FORM) /SWin:'+ IntToStr(aWin)+
                                        ' /ETS:' +IntToStr(aETS)+
                                        ' /FCN:' +aFCN+
                                        ' /File:'+aFile);
   end else
   begin  // ReEmbedded Form/Frame/DataModule, SOS for multi-level Inherited, look above Note-100
     Result:=(xRE_EMBEDDED_FORM=1);
  end;

   if Result then
    begin
       self.LoadedFormSourceWinNum:=aWin;
       self.LoadedFormEmbedTabSheetNum :=aETS;
       self.LoadedFormClassName:=aFCN;
       self.LoadedFormFileName:=aFile;
    end;

end;

Procedure TEmbedTabSheet.FormEditorStore;
begin
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;
  if FEditPages=nil then exit;
  if FEditPageForSource=nil then exit;
  if FFormEditor=nil then exit;
  if FFormEditor.FDesignPanel=nil then exit;
  if FFormEditor.FDesignPanel.FHasLoadedForm=false then exit;
  if FFormEditor.FDesignPanel.IniOK=false then exit;           //Must Init OK to Store
  if FFormEditor.FDesignPanel.FHasStoredForm=True then exit;

  CT_EMBEDDED_UPDATING:=True;

  EmbStoreForms.FormStore(FFormEditor.FDesignPanel.FLoadedForm);
  FFormEditor.FDesignPanel.FHasStoredForm := True;

  FEditPages.ActivePage:=FEditPageForSource;

  CT_EMBEDDED_UPDATING:=False;
end;

Procedure TEmbedTabSheet.FormEditorUnStore;
begin
  if CT_USE_EMBEDDED_DESIGNER=FALSE then Exit;
  if FEditPages=nil then exit;
  if FEditPageForSource=nil then exit;
  if FFormEditor=nil then exit;
  if FFormEditor.FDesignPanel=nil then exit;
  if FFormEditor.FDesignPanel.LoadedForm=Nil then exit;
  if FFormEditor.FDesignPanel.FHasLoadedForm=false then exit;
  if FFormEditor.FDesignPanel.FHasStoredForm=false then exit;

  CT_EMBEDDED_UPDATING:=True;

  Embed_SetFormParent(FFormEditor.FDesignPanel.FLoadedForm,FFormEditor.FDesignPanel);

  EmbStoreForms.FormUnStore(FFormEditor.FDesignPanel.FLoadedForm);
  FFormEditor.FDesignPanel.FHasStoredForm:=False;

 //After Dock/Undock LCLCocoa need this
 {$IF DEFINED(LCLCocoa)}
  FFormEditor.FDesignPanel.IniOK:=False;
 {$ENDIF}

  CT_EMBEDDED_UPDATING:=False;
end;

//====================================

initialization
  EMBEDDED_FORM_DESIGNER := DebugLogger.RegisterLogGroup('EMBEDDED_FORM_DESIGNER' {$IFDEF EMBEDDED_FORM_DESIGNER} , True {$ENDIF} );

end.


